#include<stdio.h>
#include<string.h>
#include<fcntl.h>
#include<stdlib.h>
#include<unistd.h>
#include<error.h>
#include<sys/stat.h>
#include<sys/types.h>
#include<sys/wait.h>
#include<sys/mount.h>
#include<dirent.h>
#include<pthread.h>
#include"addrec.h"
#include"addisorec.h"
#include"wrctl.h"
#include"utils.h"
#include"ecma_167.h"
#include"osta_udf.h"
#include"genrecif.h"


static char  deviceNameStr[20]={'\0'};
static uint32_t burnDiscSpeed = 1000;
static int verifyAfterBurn = 0;
static int verifyCloseBurn = 0;
static char tmpBurnImage[MAXIMAGENAME] = {'\0'};
static char *tmpDirFilename = NULL;
static int isBurningFlag = 0;
static int burningEndRet = 0;
static int CpFD2List(char *filepath,char *topath);
static void BurningImage(char *burncmd);
//discFSVer和discBurnFSVer定义在别处 ，因为udf两个版本可能要在别处使用它来区分版本
//初始化光驱
int InitDisc(char *devname)
{
	if (devname == NULL || strlen(devname) <= 0)
		return -1;
	logUdfInfo("[InitDisc] InitDisc(%s) start. \n",devname);
	setlocale(LC_ALL,"zh_CN.utf8");
	int ret = 0;
	char runCmd[100] ={'\0'};
	char *cmdOut = NULL;
	char *findStrPos = NULL;
	uint32_t cmdOutLen = 0;
	size_t readBuff = 0;
	
	cmdOutLen = 4096;
	cmdOut =(char*) calloc(1,cmdOutLen);
	
	//先检查是不是iso9660系统
	strcat(runCmd, "isoinfo -d -i ");
	strcat(runCmd, devname);
	readBuff = RunCurrCmd(runCmd,cmdOut,cmdOutLen);
    findStrPos = strstr(cmdOut,"CD-ROM is in ISO 9660 format");
	if (findStrPos != NULL)
	{
		logUdfInfo("[InitDisc] InitDisc as ISO9660. \n");
		discFSVer = 9660;
		ret = InitIsoFs(devname);
		InitIsoInfo(cmdOut);
	}
	else
	{
		memset(runCmd,0,100);
		memset(cmdOut,0,cmdOutLen);
		strcat(runCmd, "udfinfo ");
		strcat(runCmd, devname);
		readBuff = RunCurrCmd(runCmd,cmdOut,cmdOutLen);
    	findStrPos = strstr(cmdOut,"udfrev=");
		if (findStrPos != NULL)
		{
			if (strstr(cmdOut,"udfrev=1.02"))
			{
				discFSVer = 102;
				logUdfInfo("[InitDisc] InitDisc as UDF1.02. \n");
				ret = initUdfMod(devname); //初始化UDF格式
			}
			else if (strstr(cmdOut,"udfrev=2.01"))
			{
				discFSVer = 201;
				logUdfInfo("[InitDisc] InitDisc as UDF2.01. \n");
				ret = initUdfMod(devname); //初始化UDF格式
			}
			else
			{ //是不支持的UDF版本
				logUdfInfo("[InitDisc] InitDisc  unknown UDF. \n");
				discFSVer = 0;
				free(cmdOut);
				return -1;
				
			}
		}
		else
		{//是空盘或者坏盘
			ret = InitIsoFs(devname);
			if (ret == 1)
			{//空盘
				discFSVer = 0;
				logUdfInfo("[InitDisc] InitDisc blank disc. \n");
			}
			else
			{
				logUdfInfo("[InitDisc] InitDisc as UDF. \n");

				ret = initUdfMod(devname); //初始化UDF格式
				if (ret == 0)
				{//初始化udf格式成功
					if (getUdfModInitVer() == 2)
					{
						logUdfInfo("[InitDisc] InitDisc as UDF1.02. \n");
						discFSVer = 102;
					}
						
					else if (getUdfModInitVer() == 3)
					{
						logUdfInfo("[InitDisc] InitDisc as UDF2.01. \n");
						discFSVer = 201;
					}
					else
					{
						logUdfInfo("[InitDisc] InitDisc as UDF succeed, unknown UDF VERSION. \n");	
						free(cmdOut);
						return -1;
					}
				}
				else
				{
					logUdfInfo("[InitDisc] InitDisc as UDF failed, unknown disc. \n");	
					free(cmdOut);
					return -1;
				}
			}
		}
	}
	memset(deviceNameStr,0,20);
	strcpy(deviceNameStr,devname);
	free(cmdOut);

	return ret;
}
//获取光盘类型
char * GetDiscType()
{
	static const char *unknownDisc = "UNKNOWN";
	if (discFSVer == 102)
		return GetUdfDiscType();
	else if ((discFSVer == 9660) || (discFSVer == 0))
		return GetIsoDiscType();
	else
		{}
	return unknownDisc;
}
//获取光盘刻录次数
uint32_t * GetDiscBurnTimes()
{
	return getdiscinfo()->trk1_lastsession - 1;
}
//查询刻录过的盘的格式
char * GetBurnedDiscFS()
{
	static const char *UDF102 = "UDF1.02";
	static const char *UDF201 = "UDF2.01"; 
	static const char *ISO9660 = "ISO9660";
	static const char *FSNULL = "";
	char *ret = NULL;
	if (discFSVer == 102)
		ret = UDF102;
	else if (discFSVer == 201)
		ret = UDF201;
	else if (discFSVer == 9660)
		ret = ISO9660;
	else
		ret = FSNULL;
	return ret;
}
//检查光盘剩余空间
uint32_t GetDiscBlankBlock()
{
	uint32_t ret = 0;
	if (discFSVer == 102)
		ret = GetUdfBlankBlock();
	else if (discFSVer == 201)
		ret = GetUdfBlankBlock();
	else if (discFSVer == 9660 || discFSVer == 0)
		ret = GetIsoBlankBlock();

	return ret;
}
//获取光盘卷标 传入的参数需要有空间
int GetDiscLabelName(char *inStr)
{
	if (inStr == NULL || discFSVer == 0)
		return -1;
	if (discFSVer == 102)
		strncpy(inStr,getTrackLabel(),LABELNAMEMAX-1);
	else if (discFSVer == 201)
		strncpy(inStr,getTrackLabel(),LABELNAMEMAX-1);
	else if (discFSVer == 9660)
		strncpy(inStr,GetIsoTrackLabel(),LABELNAMEMAX-1);

	return 0;
}
//设置光盘卷标
int SetDiscLabelName(char *inStr)
{
	if (inStr == NULL)
		return -1;
	strncpy(getTrackLabel(),inStr,LABELNAMEMAX-1);
	strncpy(GetIsoTrackLabel(),inStr,LABELNAMEMAX-1);

	return 0;
}
//确定要刻录的格式 针对新盘
int SetBurnFS4Disc(char *burnFS)
{//只有全新刻录才允许设置刻录格式
	if (burnFS == NULL || discFSVer != 0)
		return -1;
	if (strcmp(burnFS,"UDF1.02") == 0)
		discBurnFSVer = 102;
	else if (strcmp(burnFS,"UDF2.01") == 0)
		discBurnFSVer = 201;
	else if (strcmp(burnFS,"ISO9660") == 0)
		discBurnFSVer = 9660;
	else
		discBurnFSVer = 102;
	logUdfInfo("[SetBurnFS4Disc]set discFSVer=[%u],discBurnFSVer=[%u]\n",discFSVer,discBurnFSVer);
	return 0;
}
//文件名转换 
//获取支持的刻录速度
uint32_t GetBurnSpeed(char * devname)
{ //dvd+rw-mediainfo /dev/sr0
	if (devname == NULL)
		devname = deviceNameStr;
	char runCmd[100] ={'\0'};
	char *cmdOut = NULL;
	char *findStrPos = NULL;
	uint32_t cmdOutLen = 0;
	size_t readBuff = 0;
	uint32_t ret = 0;
	
	cmdOutLen = 4096;
	cmdOut =(char*) calloc(1,cmdOutLen);
	
	//获取光盘光驱信息
	strcat(runCmd, "dvd+rw-mediainfo ");
	strcat(runCmd, devname);
	readBuff = RunCurrCmd(runCmd,cmdOut,cmdOutLen);
    findStrPos = strstr(cmdOut,"1.0x");
	if ((findStrPos = strstr(cmdOut,"1.0x")) != NULL)
		ret = 1;
	if ((findStrPos = strstr(cmdOut,"2.0x")) != NULL)
		ret = 2;
	if ((findStrPos = strstr(cmdOut,"4.0x")) != NULL)
		ret = 4;
	if ((findStrPos = strstr(cmdOut,"8.0x")) != NULL)
		ret = 8;
	if ((findStrPos = strstr(cmdOut,"16.0x")) != NULL)
		ret = 16;
	if ((findStrPos = strstr(cmdOut,"32.0x")) != NULL)
		ret = 32;
	free (cmdOut);
	return ret;
}
//设置刻录速度
uint32_t SetBurnSpeed(uint32_t burnspeed)
{
	burnDiscSpeed = burnspeed;
	return burnDiscSpeed;
}
//设置刻录是否校验
int SetVerifyAfterBurn(int isverify)
{
	verifyAfterBurn = isverify;
	return isverify;
}
//设置是否封盘
int SetCloseAfterBurn(int isclose)
{
	verifyCloseBurn = isclose;
	return isclose;
}
//确认需要添加的文件路径 只加入到链表中 等处理镜像的时候再真正的加入
int AddFileDirInfo(char *filepath,char* topath,int isrecorded,int isdir)
{ //假设topath都是./开头的
	return AddFileDir2List (filepath,topath,isrecorded,isdir,&stPreFileList);

}
//获取镜像大小字节
uint64_t GetBurnImgSize()
{
	uint64_t ret = 0;
	ret = (stPreFileList.allAddFileSizeBlock + (stPreFileList.dirNum * 2 + stPreFileList.fileNum)) * DVDBLOCKS + 50000000;
	return ret;
}

static int CpFD2List(char *filepath,char *topath)
{//filepath最后一位不能是/和*
	int ret = 0;
	DIR *currDir = NULL;
	struct dirent *currDirst = NULL;
	
	currDir = opendir(filepath);
	if (currDir == NULL)
	{
		logUdfError("[CpFD2List] filepath %s open failed. \n",filepath);
		return ADDFOLDERERROR;
	}
	currDirst = readdir(currDir); //readdir的返回值是静态分配，无法free
	if (currDirst == NULL)
	{
		closedir(currDir);
		return ADDFOLDERERROR;
	}

	while (currDirst != NULL)
	{
		logUdfInfo("[CpFD2List] currDirst->d_name %s  \n",currDirst->d_name);
		if (strcmp(currDirst->d_name,".") == 0) //本文件夹 跳过这个
		{}
		else if (strcmp(currDirst->d_name,"..") == 0) //上级文件夹 跳过这个
		{}
		else if(strcmp(currDirst->d_name,".") != 0 && currDirst->d_type == DT_DIR )
		{//读取到一个文件夹，需要递归处理
			char *tmpDirPath;
			char *currDirToPath = NULL;
			tmpDirPath = calloc (strlen(filepath) + strlen(currDirst->d_name) + 3, 1);
			tmpDirPath = strcat(tmpDirPath,filepath);
			tmpDirPath = strcat(tmpDirPath,"/");
			tmpDirPath = strcat(tmpDirPath,currDirst->d_name);
			AddFileDirInfo(tmpDirPath,topath,0,1); //要先创建这个文件夹
			//处理这个文件夹内的文件
			currDirToPath = calloc (strlen(topath) + strlen(tmpDirPath) + 3, 1);
			currDirToPath = strcat(currDirToPath,topath);
			currDirToPath = strcat(currDirToPath,currDirst->d_name);
			currDirToPath = strcat(currDirToPath,"/");
			logUdfInfo("[CpFD2List] CpFD2List dir name=[%s] \n",tmpDirPath);
			CpFD2List(tmpDirPath,currDirToPath);
			free (tmpDirPath);
			free(currDirToPath);
			currDirToPath = NULL;
		}
		else if(currDirst->d_type == DT_REG)
		{//读取到一个文件
			char *tmpFilePath;
			tmpFilePath = calloc (strlen(filepath) + strlen(currDirst->d_name) + 3, 1);
			tmpFilePath = strcat(tmpFilePath,filepath);
			tmpFilePath = strcat(tmpFilePath,"/");
			tmpFilePath = strcat(tmpFilePath,currDirst->d_name);
			logUdfInfo("[CpFD2List] AddFileDirInfo file name=[%s] \n",tmpFilePath);
			AddFileDirInfo(tmpFilePath,topath,0,0); //要先创建这个文件夹
			free (tmpFilePath);
		}
		else //不支持的文件类型
		{}
		currDirst = readdir(currDir);
	}
	
	if (currDir != NULL)
	{
		closedir(currDir);
		currDir = NULL;
	}

	return ret;
}
//把指定目录中的文件和文件夹都写到列表里
int CopyOnlyDirFiles(char *dirname)
{
	tmpDirFilename = dirname;
	if (dirname[strlen(dirname)-1] == '/')
	{
		dirname[strlen(dirname)-1] = '\0';
	}
	CpFD2List(dirname,"./");
	return 0;
}

//开始构成镜像
int MkImg4Burn(char *imgpath)
{
	logUdfInfo("[MkImg4Burn] discFSVer=[%f] discBurnFSVer=[%f]\n",discFSVer,discBurnFSVer);
	if (discFSVer != 0)
		discBurnFSVer = discFSVer;
	memset(tmpBurnImage,0,MAXIMAGENAME);
	if (discBurnFSVer == 9660 && discFSVer == 0)
	{//ISO9660全新刻录
		MkIsoImg(deviceNameStr,imgpath,0,1,1);
		strcpy(tmpBurnImage,GetTmpIsoName());
		
		FinIsoFs();
		
	}
	else if (discBurnFSVer == 9660 && discFSVer == 9660)
	{//ISO9660追加刻录
		MkIsoImg(deviceNameStr,imgpath,1,1,1);
		strcpy(tmpBurnImage,GetTmpIsoName());
		FinIsoFs();
		
	}
	else if (discBurnFSVer == 102 && discFSVer == 0)
	{//udf1.02全新刻录
		MkBlankUDFImage(deviceNameStr,imgpath,102);
		strcpy(tmpBurnImage,getUDFImageName());
	
		FinUdfNewBurn();
		
	}
	else if (discBurnFSVer == 102 && discFSVer == 102)
	{//udf1.02追加刻录 已经初始化过了 initUdfMod(inputfile);
		initStRootDirTree();
		WriteFL2stTree();
		updateFileTree();
		makeTmpFile(imgpath,102);
		updateISO();
//		finUdfMod(); //刻录开始之前清理初始化内容
//		FinUdfNewBurn();
		strcpy(tmpBurnImage,getUDFImageName());
	}
	else if (discBurnFSVer == 201 && discFSVer == 0)
	{//udf2.01全新刻录
		MkBlankUDFImage(deviceNameStr,imgpath,201);
		strcpy(tmpBurnImage,getUDFImageName());
		FinUdfNewBurn();
		

	}
	else if (discBurnFSVer == 201 && discFSVer == 201)
	{//udf2.01追加刻录
		initStRootDirTree();
		WriteFL2stTree();
		
		updateFileTree();
		makeTmpFile(imgpath,201);
		updateISO();
//		finUdfMod(); //刻录开始之前清理初始化内容
//		FinUdfNewBurn();
		strcpy(tmpBurnImage,getUDFImageName());
	}
	logUdfInfo("[MkImg4Burn] tmpBurnImage = [%s]\n",tmpBurnImage);
	return 0;
}
//获取待刻录文件名称
char *GettmpBurnImageName()
{
	return tmpBurnImage;
}
//获取刻录进度
int GetBurnRate()
{
	int ret = 0;
	float fret = 0;
	FILE *tmpfp;
	if (isBurningFlag == 100)
		return 100;
	if (isBurningFlag < 0)
		return isBurningFlag;
	char logname[MAXIMAGENAME+10]={'\0'};
	strcat(logname,tmpBurnImage);
	strcat(logname,".burnlog");
	tmpfp = fopen(logname, "r");

	if (tmpfp != NULL)
	{
		char buf[500]={'\0'};
		char buf2[10]={'\0'};
		char* tmpStr;
		char* tmpStr2;
		while (fgets(buf,500,tmpfp))
		{
	
			tmpStr = strchr(buf, '(');
			tmpStr2 = strchr(buf, '%');
			//printf("[GetBurnRate] tmpStr=[%s].\n",tmpStr);
			//printf("[GetBurnRate] tmpStr2=[%s].\n",tmpStr2);
			if (tmpStr != NULL && tmpStr2 != NULL && ((tmpStr2 - tmpStr) < 9))
			{
				char *erstr;
				memcpy(buf2,tmpStr + 1,tmpStr2 - tmpStr - 1);
				fret = strtof(buf2,&erstr);
			}
			memset(buf,0,500);
			memset(buf2,0,10);
		}
		ret = (int) fret;
		fclose(tmpfp);
	}
	printf("[GetBurnRate] return rate =[%d].\n",ret);
	return ret;
}
//刻录阻塞函数
static void BurningImage(char *burncmd)
{
	
	pid_t ret;
	int gisoret;
	ret = system(burncmd);
	printf("[BurningImage] system finished ret =[%d].\n",ret);
	if (ret == -1)
	{ //system执行失败 (系统资源不足)
		isBurningFlag = -1;
		printf("[BurningImage] burn failed ret = -1 .\n");
	}
	if (WIFEXITED(ret) == 0)
	{// growisofs运行异常 (刻录进程异常终止)
		isBurningFlag = -2;
		printf("[BurningImage] burn failed WIFEXITED(status) == 0.\n");
	}
	gisoret = WEXITSTATUS(ret);
	if (gisoret == 0)
	{
		isBurningFlag = 100;
		printf("[BurningImage] burned end .\n");
	}
	else if (gisoret == 150)
	{// growisofs运行异常 (没有安装改版的growisofs ?)
		isBurningFlag = -3;
		printf("[BurningImage] gisoret == 150 , may be growisofs dont support -R .\n");	
	}
	else if (gisoret == 22)
	{// growisofs运行异常 (-Z-R参数错误)
		isBurningFlag = -4;
		printf("[BurningImage] gisoret == 22 , -Z-R wrong way .\n");	
	}
	else
	{// growisofs运行异常 (-Z-R参数错误)
		isBurningFlag = -4;
		printf("[BurningImage] gisoret == 22 , -Z-R wrong way .\n");	
	}

	pthread_exit(NULL);
	return ;
}
//开始刻录
int BurnImage2Disc(char *imagename,char *devname,int addflag,int blockflag)
{
	static char *burncmd = NULL;
	if (isBurningFlag == 1)
		return -1;
	isBurningFlag = 1;
	if (burncmd == NULL)
		burncmd = calloc(MAXIMAGENAME*2 + 10,1);
	else
		memset(burncmd,0,MAXIMAGENAME*2 + 10);
	char *strspeed;
	pthread_t burnth;
	strspeed = calloc(100,1);
	if (addflag)
	{//追加刻录
		strcat(burncmd,"growisofs -R ");
	}
	else
	{//全新刻录
		printf("[BurnImage2Disc] getdiscinfo()->trk1_lastsession = [%u]\n",getdiscinfo()->trk1_lastsession);
		if(getdiscinfo()->trk1_lastsession > 1)
			strcat(burncmd,"growisofs -R ");
		else
			strcat(burncmd,"growisofs -Z ");
	}
		strcat(burncmd,devname);
		strcat(burncmd,"=");
		strcat(burncmd,imagename);

	strcat(burncmd," -speed=");
	if (burnDiscSpeed == 0)
		burnDiscSpeed = 1000;
	sprintf(strspeed,"%u",burnDiscSpeed);
	strcat(burncmd,strspeed);
	//strcat(burncmd," | tee ");
	strcat(burncmd," > ");
	strcat(burncmd,imagename);
	strcat(burncmd,".burnlog");
	
	logUdfInfo("[BurnImage2Disc] burncmd=[%s]\n",burncmd);
	burnth = pthread_create(&burnth,NULL,BurningImage,burncmd);
	if (blockflag == 1)
	{
		pthread_join(burnth,NULL);
	}
	else if  (blockflag == 0)
	{
		pthread_detach(burnth);
	}
	
	
	free(strspeed);

	return 0;
}

//清除保存的状态 弹出光盘后用这个
int ClearDiscBurnInfo()
{
	logUdfInfo("[ClearDiscBurnInfo] discFSVer=[%f] discBurnFSVer=[%f]\n",discFSVer,discBurnFSVer);
	
	if (tmpBurnImage != NULL)
	{	char delcmd[MAXIMAGENAME]={'\0'};
		
		strcat(delcmd,"rm -rf ");
		strcat(delcmd,tmpBurnImage);
		strcat(delcmd,"*"); //把日志文件一起删了
		system(delcmd);

	}
	if (tmpDirFilename != NULL)
	{
		char delcmdex[MAXIMAGENAME]={'\0'};
		strcat(delcmdex,"rm -rf \"");
		strcat(delcmdex,tmpDirFilename);
		strcat(delcmdex,"\"");
		system(delcmdex);
		tmpDirFilename = NULL;
	}
	memset(tmpBurnImage,0,MAXIMAGENAME);
	if (discBurnFSVer == 9660 && discFSVer == 0)
	{//ISO9660全新刻录
		FinIsoFs();
	}
	else if (discBurnFSVer == 9660 && discFSVer == 9660)
	{//ISO9660追加刻录
		FinIsoFs();
	}
	else if (discBurnFSVer == 102 && discFSVer == 0)
	{//udf1.02全新刻录
		FinUdfNewBurn();
	}
	else if (discBurnFSVer == 102 && discFSVer == 102)
	{//udf1.02追加刻录
		FinUdfNewBurn();
		finUdfMod(); //刻录开始之前清理初始化内容
	}
	else if (discBurnFSVer == 201 && discFSVer == 0)
	{//udf2.01全新刻录
		FinUdfNewBurn();
	}
	else if (discBurnFSVer == 201 && discFSVer == 201)
	{//udf2.01追加刻录
		FinUdfNewBurn();
		finUdfMod(); //刻录开始之前清理初始化内容
	}
	discBurnFSVer = 0;
	discFSVer = 0;
//	isBurningFlag = 0;
	return 0;
}
//在这里设置可以在读不出来格式的时候指定轨道和格式
int InitWithOverride(char *devname, double cdiscFSVer, uint32_t ccurrTrack)
{
	setlocale(LC_ALL,"zh_CN.utf8");
	discFSVer = cdiscFSVer;
	currTrack = ccurrTrack;
	initUdfMod(devname); //初始化UDF格式

	memset(deviceNameStr,0,20);
	strcpy(deviceNameStr,devname);
}
#if PATCH_HAVE_IF
int main (int argc, char *argv[])
{
    printf ("argc=[%d]\n",argc);
	
	char *inputfile,*outputfile;
	struct generic_desc *p;
	const char *dirnamexx = "xss";
	const char *dirpathxx = "./mcmods";
	if (argv[1][0] == '-')
	{
	
		if (argv[1][1] == 'E')
		{	
			int ifstartpos,ofstartpos,numbytes;
			if (argc < 3)
			{
				printf ("err: argc < 3\n");
				printf ("-A for replace example:  -A /dev/sr0 \n");
				return 0;
			}
			else
			{
				inputfile = argv[2];
            }
        }
		else if (argv[1][1] == 'R')
		{	
			int ifstartpos,ofstartpos,numbytes;
			inputfile = argv[2];
			InitDisc(inputfile);
			
			printf ("GetDiscType()=%s\n",GetDiscType());
			printf ("GetDiscBurnTimes()=%u\n",GetDiscBurnTimes());
			printf ("GetBurnedDiscFS()=%s\n",GetBurnedDiscFS());
			printf ("GetDiscBlankBlock()=%u\n",GetDiscBlankBlock());
			char labelstr[257]={'\0'};
			
			GetDiscLabelName(labelstr);
			printf ("GetDiscLabelName()=%s\n",labelstr);
			SetBurnFS4Disc("ISO9660");
			uint32_t burnspeed;
			burnspeed = GetBurnSpeed(inputfile);
			printf ("GetBurnSpeed()=%u\n",burnspeed);
			SetBurnSpeed(1000);
			SetVerifyAfterBurn(0);
			SetCloseAfterBurn(0);
			printf ("AddFileDirInfo() start.\n");
			AddFileDirInfo("/home/nfs/work/DE/20220606dvd/udf201.pdf","./",0,0);
			MkImg4Burn("/home/nfs/cdr");
			printf ("GetBurnImgSize()=%u\n",GetBurnImgSize());
			ClearDiscBurnInfo();
            
        }
		else if (argv[1][1] == 'T')
		{	
			int ifstartpos,ofstartpos,numbytes;
			inputfile = argv[2];
			InitDisc(inputfile);
			
			printf ("GetDiscType()=%s\n",GetDiscType());
			printf ("GetDiscBurnTimes()=%u\n",GetDiscBurnTimes());
			printf ("GetBurnedDiscFS()=%s\n",GetBurnedDiscFS());
			printf ("GetDiscBlankBlock()=%u\n",GetDiscBlankBlock());
			char labelstr[257]={'\0'};
			
			GetDiscLabelName(labelstr);
			printf ("GetDiscLabelName()=%s\n",labelstr);
			SetBurnFS4Disc("UDF1.02");
			//SetBurnFS4Disc("UDF2.01");
			SetDiscLabelName("mydr");
			uint32_t burnspeed;
			burnspeed = GetBurnSpeed(inputfile);
			printf ("GetBurnSpeed()=%u\n",burnspeed);
			SetBurnSpeed(1000);
			SetVerifyAfterBurn(0);
			SetCloseAfterBurn(0);
			printf ("[main] CopyOnlyDirFiles() AddFileDirInfo() start .\n");
			CopyOnlyDirFiles("/home/nfs/m1");
			AddFileDirInfo("/home/nfs/work/DE/20220606dvd/编译k3b.txt","./",0,0);
			AddFileDirInfo("/home/nfs/work/DE/20220606dvd/ECMA.txt","./",0,0);
			AddFileDirInfo("/home/nfs/cdr/40gfile","./",0,0);
			AddFileDirInfo("/home/nfs/media/[AJzero&Blusub][Rakuen Tsuiho][Movie][GB_BIG5][1080P][264_FLAC].mkv","./",0,0);
			MkImg4Burn("/home/nfs/cdr");
			printf ("GetBurnImgSize()=%u\n",GetBurnImgSize());
			ClearDiscBurnInfo();
            
        }
		else if (argv[1][1] == 'Y')
		{	
			int ifstartpos,ofstartpos,numbytes;
			inputfile = argv[2];
		
			//InitWithOverride(inputfile,102,0);

			InitDisc(inputfile);
			printf ("GetDiscType()=%s\n",GetDiscType());
			printf ("GetDiscBurnTimes()=%u\n",GetDiscBurnTimes());
			printf ("GetBurnedDiscFS()=%s\n",GetBurnedDiscFS());
			printf ("GetDiscBlankBlock()=%u\n",GetDiscBlankBlock());
			char labelstr[257]={'\0'};
			
			GetDiscLabelName(labelstr);
			printf ("GetDiscLabelName()=%s\n",labelstr);
			SetBurnFS4Disc("UDF1.02");
			SetDiscLabelName("mydr");
			uint32_t burnspeed;
			burnspeed = GetBurnSpeed(inputfile);
			printf ("GetBurnSpeed()=%u\n",burnspeed);
			SetBurnSpeed(1000);
			SetVerifyAfterBurn(0);
			SetCloseAfterBurn(0);
			
			//AddFileDirInfo("/home/nfs/work/DE/20220606dvd/udf201.pdf","./",0,0);
			AddFileDirInfo("/home/nfs/work/DE/20220606dvd/ECMA.txt","./",0,0);
			//AddFileDirInfo("/home/nfs/work/DE/20220606dvd/编译k3b.txt","./",0,0);
			//AddFileDirInfo("/home/nfs/cdr/5gfile","./",0,0);
			//AddFileDirInfo("/home/nfs/media/[AJzero&Blusub][Rakuen Tsuiho][Movie][GB_BIG5][1080P][264_FLAC].mkv","./",0,0);
			
			
			MkImg4Burn("/home/nfs/cdr");
			printf ("GetBurnImgSize()=%u\n",GetBurnImgSize());
			ClearDiscBurnInfo();
		}
		else if (argv[1][1] == 'U')
		{	
			int ifstartpos,ofstartpos,numbytes;
			inputfile = argv[2];
			// /home/nfs/cdr/xxudf /home/nfs/cdr/cdx  还是需要权限，放弃这种办法
			printf ("start unicode\n");
			setlocale(LC_ALL,"zh_CN.utf8");
			char *tmpStr = "编译k3b.txt";
			dchars *uName;
			char name[FNAMELENMAX] = {'\0'};
			uName = calloc (FNAMELENMAX, 1);
					size_t uLen;
		logUdfInfo("[main] utf8_TO_utf16 strlen tmpStr=[%u] \n",strlen(tmpStr));
		logUdfInfo("[main] utf8_TO_utf16 sizeof tmpStr=[%u] \n",sizeof(tmpStr));
		logUdfInfo("[main] utf8_TO_utf16 count_utf8_bytes tmpStr=[%u] \n",count_utf8_bytes(tmpStr));
		
			uLen = utf8_TO_utf16(uName + 1, tmpStr, 127);
			logUdfInfo("[main] utf8_TO_utf16 tmpStr uLen=[%u] \n",uLen);
			uName[0] = 16;
			
			uLen++;
			logPrintX(uName,uLen);
			uLen = decode_locale((dchars *)(uName), name, uLen, FNAMELENMAX - 1);
			//logUdfInfo("[main] decode_locale tmpST->name, uLen=[%u] \n",uLen);
			free(uName);
			logUdfInfo("[main] name=[%s] \n",name);

			
		}
		else if (argv[1][1] == 'I')
		{	
			int ifstartpos,ofstartpos,numbytes;
			inputfile = argv[2];
			// /home/nfs/cdr/xxudf /home/nfs/cdr/cdx  
			printf ("start CopyOnlyDirFiles\n");
			CopyOnlyDirFiles("/home/nfs/m1");
			
		}
		else if (argv[1][1] == 'O')
		{	
			int ifstartpos,ofstartpos,numbytes;
			inputfile = argv[2];
			int ret=0;
			int i=0;
			i = 10;
			strcpy(tmpBurnImage,"/home/nfs/m3/isonew144020.udf");
			printf ("start GetBurnRate\n");
			while(i--)
			{
				sleep(1);
				GetBurnRate();
			}
			
			
		}
		// test  GetBurnRate
		else if (argv[1][1] == '9')
		{	
			int ifstartpos,ofstartpos,numbytes;
			inputfile = argv[2];
			// /home/nfs/cdr/xxudf /home/nfs/cdr/cdx  还是需要权限，放弃这种办法
			printf ("start mount\n");
			mount("/home/nfs/cdr/xxudf","/home/nfs/cdr/cdx","udf",MS_RDONLY,NULL);
			("sudo ls /home/nfs/cdr/cdx >/home/nfs/cdr/ls.log");
			printf ("start umount\n");
			umount("/home/nfs/cdr/xxudf");
			
		}
    }
    return 0;
}
#endif


