#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include "wrctl.h"
#include "ecma_167.h"
#include "defaultvar.h"
#include "utils.h"

#include <stdarg.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/types.h> /* for u_char etc. */
#include <linux/cdrom.h>
#include <unistd.h> /* sleep() */
#include <stdlib.h>

#include <scsi/scsi.h>
#include <scsi/sg.h>

typedef struct cdrom_generic_command CGC;
static int rv; // -1 if ioctl failed, else equal stat
static struct request_sense sense_data;
static void initpc(CGC *pc);

struct _Read10CDB
{
    unsigned char opcode;

    unsigned char reladr : 1;
    unsigned char reserved1 : 2;
    unsigned char FUA : 1;
    unsigned char DPO : 1;
    unsigned char reserved2 : 3;

    uint32_t start_address;

    unsigned char reserved3;

    uint16_t len;

    unsigned char ctl;
};

static void initpc(CGC *pc)
{
    memset(pc, 0, sizeof(*pc));
    memset(&sense_data, 0, sizeof(sense_data));
    pc->sense = &sense_data;
    pc->data_direction = CGC_DATA_NONE;
    pc->timeout = 2500;
}

int read_discinfo(int fd, struct cdrom_discinfo *di)
{
    CGC pc;
    uint16_t size_be16 = cpu_to_be16(sizeof(struct cdrom_discinfo));

    initpc(&pc);
    pc.data_direction = CGC_DATA_READ;

    pc.cmd[0] = GPCMD_READ_DISC_INFO;
    memcpy(&pc.cmd[7], &size_be16, sizeof(size_be16));
    pc.buffer = (void *)di;
    pc.buflen = sizeof(struct cdrom_discinfo);

    rv = ioctl(fd, CDROM_SEND_PACKET, &pc);

    di->data_length = be16_to_cpu(di->data_length);
    di->disc_id = be32_to_cpu(di->disc_id);
    return rv;
}

int read_trackinfo(int fd, struct cdrom_trackinfo *ti, int trackno)
{
    CGC pc;
    uint32_t trackno_be32 = cpu_to_be32(trackno);
    uint16_t size_be16 = cpu_to_be16(sizeof(struct cdrom_trackinfo));

    initpc(&pc);
    pc.data_direction = CGC_DATA_READ;
    pc.cmd[0] = GPCMD_READ_TRACK_RZONE_INFO;
    pc.cmd[1] = 1;
    memcpy(&pc.cmd[2], &trackno_be32, sizeof(trackno_be32));
    memcpy(&pc.cmd[7], &size_be16, sizeof(size_be16)); /* 28; if higher data underrun error */
    pc.buffer = (void *)ti;
    pc.buflen = sizeof(struct cdrom_trackinfo);
    rv = ioctl(fd, CDROM_SEND_PACKET, &pc);
    ti->data_length = be16_to_cpu(ti->data_length);
    ti->trk_start = be32_to_cpu(ti->trk_start);
    ti->nwa = be32_to_cpu(ti->nwa);
    ti->free_blks = be32_to_cpu(ti->free_blks);
    ti->fixpkt_size = be32_to_cpu(ti->fixpkt_size);
    ti->trk_size = be32_to_cpu(ti->trk_size);
    return rv;
}

int readCD(int fd, int sectortype, int lba, int n, unsigned char *buf)
{
    CGC pc;
    uint32_t lba_be32 = cpu_to_be32(lba);

    initpc(&pc);
    pc.data_direction = CGC_DATA_READ;
    pc.cmd[0] = GPCMD_READ_CD;
    pc.cmd[1] = sectortype << 2; /* Sector type per MCC table 31 */
    memcpy(&pc.cmd[2], &lba_be32, sizeof(lba_be32));
    pc.cmd[8] = n;    /* read n blocks */
    pc.cmd[9] = 0x10; /* user data only */
    pc.buffer = buf;
    pc.buflen = n * 2048;
    return rv = ioctl(fd, CDROM_SEND_PACKET, &pc);
}

int transportSGIO(int fd, unsigned char *cmd, unsigned char cmd_len, int direction, unsigned char *buf, size_t sz)
{
    struct sg_io_hdr sg_io;
    unsigned char sbp_data[SBPDATAMAXLEN];
    memset(&sg_io, 0, sizeof(sg_io));
    sg_io.interface_id = 'S';
    sg_io.mx_sb_len = SBPDATAMAXLEN;

    sg_io.sbp = sbp_data;
    sg_io.flags = SG_FLAG_LUN_INHIBIT | SG_FLAG_DIRECT_IO;

    sg_io.cmdp = cmd;
    sg_io.cmd_len = cmd_len;
    sg_io.dxfer_direction = direction;
    sg_io.dxferp = buf;
    sg_io.dxfer_len = sz;
    rv = ioctl(fd, SG_IO, &sg_io);
    if (rv)
    {
        printf("[transportSGIO] errno = %d , strerror = %s \n", errno, strerror(errno));
        return rv;
    }
    return rv;
}

int readMediaMMCTYPE(int fd)
{
    unsigned char buf[128], cmd[12];

    cmd[0] = 0x12; // INQUIRY
    cmd[4] = 36;
    cmd[5] = 0;

    transportSGIO(fd, cmd, 12, SG_DXFER_FROM_DEV, buf, 36);
    if ((buf[0] & 0x1F) != 5)
    {
        printf("[readMediaMMC]:-( not an MMC unit!\n");
        return 0;
    }
    return 0;
}

int openMediaSGIO(int fd)
{
    unsigned char buf[128], cmd[12];

    cmd[0] = 0x1B; // START/STOP UNIT
    cmd[1] = 0x1;  // "IMMED"
    cmd[4] = 0x3;  // "Load"
    cmd[5] = 0;

    transportSGIO(fd, cmd, 12, SG_DXFER_FROM_DEV, NULL, 0);

    return 0;
}

int readCDex(int fd, int sectortype, int lba, int n, unsigned char *buf)
{
    unsigned char cmd[10];
    uint32_t ulba;
    ulba = cpu_to_le32(lba) ;
    
    cmd[0] = 0x28; // READ(10)
    cmd[2] = ulba >> 24;
    cmd[3] = ulba >> 16;
    cmd[4] = ulba >> 8;
    cmd[5] = ulba;
    cmd[7] = 0;
    cmd[8] = 1;    //还是不确定怎么知道一个block是2048的
    cmd[9] = 0;
//    printf("[readCDex] lba = %d , n = %d \n",lba,n);

    return transportSGIO(fd, &cmd, 10, SG_DXFER_FROM_DEV, buf, DVDBLOCKS);
}

int readFileBlock(int *file, uint32_t afterBlkno, struct generic_desc *p)
{
    long offst;
    offst = afterBlkno;
    offst = offst * DVDBLOCKS;
    fseek(file, offst, SEEK_SET);
    if (fread(p, 1, DVDBLOCKS, file) < DVDBLOCKS)
    {
        logUdfError("[readFileBlock] err: read end \n");
        return -1;
    }
    return 0;
}

int writeFileBlock(int *file, uint32_t afterBlkno, struct generic_desc *p, size_t writeByte)
{
    long offst;
    offst = afterBlkno;
    offst = offst * DVDBLOCKS;
    if (fseek(file, offst, SEEK_SET) != 0)
    {
        logUdfError("[writeFileBlock] fseek err,afterBlkno=%u\n", afterBlkno);
        return -1;
    }
    else
    {
        size_t writeRet;
        writeRet = fwrite(p, 1, writeByte, file);
        if (writeRet != writeByte)
        {
            logUdfError("[writeFileBlock] fwrite err,writeByte=%u,writeRet=%u\n", writeByte, writeRet);
            return -1;
        }
    }

    return 0;
}

int iodevctl()
{
    return 0;
}
