#ifndef _WRCTL_H
#define _WRCTL_H

#include<stdio.h>
#include"ecma_167.h"

#include <sys/types.h>

//defaultvar作为本.h文件的延伸
#include"defaultvar.h"

#define PATCH_USE_SG_IO
#define DVDBLOCKS 2048
#define PKGLEN 16
#define DISK_IMAGE	0xAA
#define MAXDIRSTRALONG 6000
#define MAXIMAGENAME 256

#define ABSOLUTE	0xFFFF				/* ignore partition, process physical blocknumber */

#define FLAG_FREED_BITMAP		0x00000001
#define FLAG_FREED_TABLE		0x00000002
#define FLAG_UNALLOC_BITMAP		0x00000004
#define FLAG_UNALLOC_TABLE		0x00000008
#define FLAG_SPACE_BITMAP		(FLAG_FREED_BITMAP|FLAG_UNALLOC_BITMAP)
#define FLAG_SPACE_TABLE		(FLAG_FREED_TABLE|FLAG_UNALLOC_TABLE)
#define FLAG_SPACE			(FLAG_SPACE_BITMAP|FLAG_SPACE_TABLE)

#define FLAG_LOCALE			0x00000010
#define FLAG_UNICODE8			0x00000020
#define FLAG_UNICODE16			0x00000040
#define FLAG_UTF8			0x00000080
#define FLAG_CHARSET			(FLAG_LOCALE|FLAG_UNICODE8|FLAG_UNICODE16|FLAG_UTF8)

#define FLAG_STRATEGY4096		0x00000100
#define FLAG_BLANK_TERMINAL		0x00000200

#define FLAG_CLOSED			0x00000800
#define FLAG_VAT			0x00001000

#define FLAG_EFE			0x00002000

#define FLAG_NO_WRITE			0x00004000

#define FLAG_BOOTAREA_PRESERVE		0x00010000
#define FLAG_BOOTAREA_ERASE		0x00020000
#define FLAG_BOOTAREA_MBR		0x00040000
#define FLAG_BOOTAREA_MASK		(FLAG_BOOTAREA_PRESERVE|FLAG_BOOTAREA_ERASE|FLAG_BOOTAREA_MBR)

#define SBPDATAMAXLEN		19

enum RV { CMND_OK, CMND_FAILED, WRONG_NO_ARGS, CMND_ARG_INVALID, NOT_IN_RW_PARTITION,
	  DIR_INVALID, EXISTING_DIR, EXISTING_FILE, DELETED_DIR, DELETED_FILE, DOES_NOT_EXIST,
	  DIR_NOT_EMPTY, PERMISSION_DENIED, IS_DIRECTORY };

struct generic_desc
{
	tag		descTag;
	uint32_t	volDescSeqNum;
};


struct udf_extent;
struct udf_desc;
struct udf_data;
struct stfiledir;

enum udf_space_type
{
	RESERVED	= 0x0001,	/* Reserved Space */
	VRS		= 0x0002,	/* Volume Recognition Sequence */
	ANCHOR		= 0x0004,	/* Anchor */
	MVDS		= 0x0008,	/* Main Volume Descriptor Sequence */
	RVDS		= 0x0010,	/* Reserve Volume Descriptor Sequence */
	LVID		= 0x0020,	/* Logical Volume Integrity Descriptor */
	STABLE		= 0x0040,	/* Sparing Table */
	SSPACE		= 0x0080,	/* Sparing Space */
	PSPACE		= 0x0100,	/* Partition Space */
	USPACE		= 0x0200,	/* Unallocated Space */
	BAD		= 0x0400,	/* Bad Blocks */
	MBR		= 0x0800,	/* MBR Boot Area */
	UDF_SPACE_TYPE_SIZE = 12,
};

struct udf_sizing
{
	uint32_t	align;
	uint32_t	numSize;
	uint32_t	denomSize;
	uint32_t	minSize;
};

enum udf_alloc_type
{
	VDS_SIZE,
	LVID_SIZE,
	STABLE_SIZE,
	SSPACE_SIZE,
	PSPACE_SIZE,
	UDF_ALLOC_TYPE_SIZE,
};

struct udf_disc
{
	uint16_t			udf_rev;
	uint16_t			udf_write_rev;
	uint32_t			blocksize;
	uint32_t			blocks;
	uint32_t			flags;
	unsigned int			blkssz;
	uint64_t			blksize;
	uint32_t			num_files;
	uint32_t			num_dirs;
	uint32_t			free_space_blocks;
	uint32_t			total_space_blocks;

	uint32_t			uid;
	uint32_t			gid;
	uint16_t			mode;

	struct udf_sizing		sizing[UDF_ALLOC_TYPE_SIZE];

	int				(*write)(struct udf_disc *, struct udf_extent *);
	void				*write_data;

	struct volStructDesc		*udf_vrs[3];
	struct anchorVolDescPtr		*udf_anchor[3];
	struct primaryVolDesc		*udf_pvd[2];
	struct logicalVolDesc		*udf_lvd[2];
	struct partitionDesc		*udf_pd[2];
	struct partitionDesc		*udf_pd2[2];
	struct unallocSpaceDesc		*udf_usd[2];
	struct impUseVolDesc		*udf_iuvd[2];
	struct terminatingDesc		*udf_td[2];
	struct logicalVolIntegrityDesc	*udf_lvid;

	struct sparingTable		*udf_stable[4];

	uint32_t			start_block;
	uint32_t			last_block;

	uint32_t			vat_block;
	uint32_t			*vat;
	uint64_t			vat_entries;

	short_ad			*metadata_filemap[2];
	uint32_t			metadata_filemap_count[2];

	struct fileSetDesc		*udf_fsd;

	struct udf_extent		*head;
	struct udf_extent		*tail;
};

struct udf_extent
{
	enum udf_space_type		space_type;
	uint32_t			start;
	uint32_t			blocks;

	struct udf_desc			*head;
	struct udf_desc			*tail;

	struct udf_extent		*next;
	struct udf_extent		*prev;
};

struct udf_desc
{
	uint16_t			ident;
	uint32_t			offset;
	uint64_t			length;
	struct udf_data			*data;

	struct udf_desc			*next;
	struct udf_desc			*prev;
};

struct udf_data
{
	uint64_t			length;
	void				*buffer;
	struct udf_data			*next;
	struct udf_data			*prev;
};

struct countfile
{
	uint32_t filenum;  //统计文件链表里多少个文件
	uint32_t dirnum;  //统计文件链表里多少个文件夹
	uint32_t writeblkno; //计算fe写入的位置blocks
	uint32_t contentstartblkno;  //计算文件内容写入的位置blocks
	uint32_t offset;  //有的文件夹内容过多，需要后移写入位置
};

struct stprefiledir
{
	char *filePath; //文件在系统里的绝对路径
	char *toPath; //文件要放到光盘内的相对.(根目录)的路径
	int isRecorded; //是否已经刻录过的内容
	int isDir; //是否是文件夹
	struct stprefiledir *prev;
	struct stprefiledir *next;
};

struct stprefiledirlist
{
	struct stprefiledir *fdStart; //文件文件夹链表第一个数据
	struct stprefiledir *fdEnd; //文件文件夹列表最后一个数据
	uint32_t fileNum; //目前列表里的文件数量
	uint32_t dirNum; //目前列表里的文件夹数量
	uint64_t allAddFileSizeBlock;  //全部待刻录文件的大小
};

//在ecma_167.h中定义了一个stfiledir结构体，因为要对外部接口，只能放在别的.h中

typedef int (*fpstFD)(struct stfiledir *) ;
/* crc.c */
extern uint16_t udf_crc(uint8_t *, uint32_t, uint16_t);

/* unicode.c */
extern size_t decode_utf8(const dchars *, char *, size_t, size_t);
extern size_t encode_utf8(dchars *, const char *, size_t);
extern size_t decode_locale(const dchars *, char *, size_t, size_t);
extern size_t encode_locale(dchars *, const char *, size_t);
extern size_t decode_string(struct udf_disc *, const dstring *, char *, size_t, size_t);
extern size_t encode_string(struct udf_disc *, dstring *, const char *, size_t);
extern uint8_t get_utf8_bytes(uint8_t utf8);
extern uint16_t utf8_TO_utf16(uint16_t *out, uint8_t *in, uint16_t outLen);
extern uint16_t count_utf8(uint8_t *in);
uint32_t count_utf8_bytes(uint8_t *in);
//ide.h


//filectl.c

struct fileIdentDesc*	findFileIdentDesc(Directory *dir, char* name);
int			deleteFID(Directory *dir, struct fileIdentDesc *fid);
int			removeFID(Directory *dir, struct fileIdentDesc *fid);
int			insertFileIdentDesc(Directory *dir, struct fileIdentDesc* fid);
struct fileEntry*	makeFileEntry();
struct extendedFileEntry*	makeFileEntryEx();

Directory * readDirectory(Directory *parentDir, long_ad* icb, char *name);
int lscCommand(void);
enum RV analyzeDest(char* arg, struct fileIdentDesc** fid, char** name);

struct fileIdentDesc* findFileIdentDesc(Directory *dir, char* name);



int updateFePermissions(struct fileEntry* fe, char* filepath);
int updateFePermissionsEx(struct extendedFileEntry* efe, char* filepath);
struct fileIdentDesc*	makeFileIdentDesc(char* name);

struct stfiledir *delNode4Tree(char* filepath);
char *cutPathName(char* filepath,char* retName);
int copyFile2Tree(char* filepath, char* topath, struct stat *st);
int linkNode2Tree(struct stfiledir *newst, char* topath);
int makeDir2Tree(char* dirname, char* topath);
int AddFileDir2List(char *filepath,char* topath,int isrecorded,int isdir,struct stprefiledirlist *stpfdl);
int DelFileDirList(struct stprefiledirlist *stpfdl);
char *DealWithBlankCmd(char *instr);
//ioctl.c
int read_discinfo(int fd, struct cdrom_discinfo *di);
int read_trackinfo(int fd, struct cdrom_trackinfo *ti, int trackno);
int readCD(int fd, int sectortype, int lba, int n, unsigned char* buf);

int transportSGIO(int fd,unsigned char *cmd,unsigned char cmd_len,int direction,unsigned char *buf,size_t sz);
int readMediaMMCTYPE(int fd);
int openMediaSGIO(int fd);
int readCDex(int fd, int sectortype, int lba, int n, unsigned char* buf);

int readFileBlock(int *file, uint32_t afterBlkno, struct generic_desc *p);
int writeFileBlock(int *file, uint32_t afterBlkno, struct generic_desc *p, size_t writeByte);
//wrctl.c

extern Directory *rootDir, *curDir; //因为用法太多太特殊必须设置的全局变量
extern uint32_t *vat;
extern uint32_t *multiStart;
extern uint32_t addIsoBlocks;
extern uint16_t virtualPartitionNum;
extern volatile struct stfiledir *stRootDir;
extern struct stprefiledirlist  stPreFileList;
extern timestamp	timeStamp;
extern double discFSVer;
extern double discBurnFSVer;
extern uint32_t currTrack;
//extern FILE* pfile;
int initUdfDev(char *devname);
int closeUdfDev();
void* readSingleBlock(uint32_t pbn);
void* readTaggedBlock(uint32_t lbn, uint16_t partition);
unsigned char*	readCDR(uint32_t lbn, uint16_t partition);
uint32_t	setPartitionDesc(struct partitionDesc *tmpd);
uint32_t getPhysical(uint32_t lbn, uint16_t part);

void freeBlock(uint32_t lbn, uint16_t part);
void* readBlock(uint32_t lbn, uint16_t part);
int readExtents(char* dest, int usesShort, void* extents);

int patchAddUdfFs(char *isoFileName);
int patchAddUdfFsFe(uint32_t feBlock);
void setChecksum(void *descriptor);

int initStRootDirTree();

int addDir2RootDirTree(char *dirFid, uint32_t dataLength, struct stfiledir *stDir);
int copyDirData(struct fileEntry  *fe, struct stfiledir *stDir);

int reDirTree(struct stfiledir* stDir, fpstFD cbFun);
int reDirTreePre(struct stfiledir* stDir, fpstFD cbFun);

int updateStTree();
int updateStConStart(struct stfiledir *stDir);
int updateStNode(struct stfiledir *stDir);
int updateStNodeDir(struct stfiledir *stDir);

int updateStNodeEx(struct stfiledir *stDir);
int updateStNodeDirEx(struct stfiledir *stDir);

int writeFDTree2ISO();
int writeFDNode2ISO(struct stfiledir *stDir);

int delStRootDirDescTree();
int delStNode(struct stfiledir *stDir);

int contrlfilenum(uint32_t filenum,uint32_t dirnum);
int countStNodeBlock(struct stfiledir *stDir);
int countUsedBlock();
int countStNode(struct stfiledir *stDir);
uint32_t countStNodeAddTagLocation(struct stfiledir *stDir);
int InitBlankRootsTree(char* dirname);
int writeFDTree2Image(FILE *inpath,struct stfiledir *stDir);
int clearCountFileNum();
int updateCurrStTree(struct stfiledir *stDir);




#endif


